package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.container.FFMpegCommand;
import net.knarcraft.ffmpegconverter.container.StreamProbeResult;
import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import org.jetbrains.annotations.NotNull;

import java.util.List;

/**
 * A converter for converting audio files
 */
public class AudioConverter extends AbstractConverter {

    /**
     * Instantiates a new audio converter
     *
     * @param ffprobePath  <p>Path/command to ffprobe.</p>
     * @param ffmpegPath   <p>Path/command to ffmpeg.</p>
     * @param newExtension <p>The extension of the new file.</p>
     */
    public AudioConverter(String ffprobePath, String ffmpegPath, String newExtension) {
        super(newExtension);
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
    }

    @Override
    public String[] generateConversionCommand(@NotNull String executable, @NotNull StreamProbeResult probeResult,
                                              @NotNull String outFile) {
        FFMpegCommand command = FFMpegHelper.getFFMpegGeneralFileCommand(executable, probeResult.parsedFiles());
        List<StreamObject> streams = probeResult.parsedStreams();
        if (this.debug) {
            FFMpegHelper.addDebugArguments(command, 50, 120);
        }

        //Gets the first audio stream from the file and adds it to the output file
        AudioStream audioStream = getNthAudioSteam(streams, 0);
        FFMpegHelper.addAudioStream(command, audioStream, false);
        command.setOutputFile(outFile);

        return command.getResult();
    }

    @Override
    public String[] getValidFormats() {
        return audioFormats;
    }

}