package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.container.FFMpegCommand;
import net.knarcraft.ffmpegconverter.container.StreamProbeResult;
import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.streams.VideoStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import org.jetbrains.annotations.NotNull;

import java.util.List;

/**
 * A converter for converting video files
 */
public class DownScaleConverter extends AbstractConverter {

    private final int newWidth;
    private final int newHeight;

    /**
     * Instantiates a new video converter
     *
     * @param ffprobePath <p>Path/command to ffprobe.</p>
     * @param ffmpegPath  <p>Path/command to ffmpeg.</p>
     * @param newWidth    <p>The new width of the video</p>
     * @param newHeight   <p>The new height of the video</p>
     */
    public DownScaleConverter(String ffprobePath, String ffmpegPath, int newWidth, int newHeight) {
        super(null);
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
        this.newHeight = newHeight;
        this.newWidth = newWidth;
    }

    @Override
    public String[] generateConversionCommand(@NotNull String executable, @NotNull StreamProbeResult probeResult,
                                              @NotNull String outFile) {
        List<StreamObject> streams = probeResult.parsedStreams();
        VideoStream videoStream = getNthVideoStream(streams, 0);
        if (videoStream == null || (videoStream.getWidth() <= newWidth && videoStream.getHeight() <= newHeight)) {
            return new String[0];
        }

        FFMpegCommand command = FFMpegHelper.getFFMpegGeneralFileCommand(executable, probeResult.parsedFiles());
        if (this.debug) {
            FFMpegHelper.addDebugArguments(command, 50, 120);
        }

        //Add all streams without re-encoding
        FFMpegHelper.mapAllStreams(command, streams);
        command.addOutputFileOption("-c:a", "copy");
        command.addOutputFileOption("-c:s", "copy");
        command.addOutputFileOption("-vf", "scale=" + newWidth + ":" + newHeight);
        command.addOutputFileOption("-crf", "20");
        command.addOutputFileOption("-preset", "slow");
        command.setOutputFile(outFile);
        return command.getResult();
    }

    @Override
    public String[] getValidFormats() {
        return videoFormats;
    }

}