package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.container.FFMpegCommand;
import net.knarcraft.ffmpegconverter.container.StreamProbeResult;
import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.streams.SubtitleStream;
import net.knarcraft.ffmpegconverter.streams.VideoStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import org.jetbrains.annotations.NotNull;

import java.util.List;

/**
 * A converter solely for the purpose of converting video streams of MKV files into h264
 */
public class MkvH265ReducedConverter extends AbstractConverter {

    /**
     * Initializes variables used by the abstract converter
     *
     * @param ffprobePath <p>Path/command to ffprobe.</p>
     * @param ffmpegPath  <p>Path/command to ffmpeg.</p>
     */
    public MkvH265ReducedConverter(String ffprobePath, String ffmpegPath) {
        super("mkv");
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
    }

    @Override
    public String[] getValidFormats() {
        return new String[]{"mkv"};
    }

    @Override
    public String[] generateConversionCommand(@NotNull String executable, @NotNull StreamProbeResult probeResult,
                                              @NotNull String outFile) {
        FFMpegCommand command = FFMpegHelper.getFFMpegGeneralFileCommand(executable, probeResult.parsedFiles());
        List<StreamObject> streams = probeResult.parsedStreams();

        if (this.debug) {
            FFMpegHelper.addDebugArguments(command, 50, 120);
        }

        // Map video if present
        List<StreamObject> videoStreams = filterStreamsByType(streams, VideoStream.class);
        if (!videoStreams.isEmpty()) {
            for (StreamObject streamObject : videoStreams) {
                if (!streamObject.getCodecName().equals("h264") && !streamObject.getCodecName().equals("h266")) {
                    continue;
                }

                FFMpegHelper.addH26xHardwareDecoding(command);
                break;
            }

            FFMpegHelper.mapAllStreams(command, videoStreams);
            FFMpegHelper.addH265HardwareEncoding(command, 17);
            command.addOutputFileOption("-movflags", "+faststart");
        }

        // Map audio if present
        List<StreamObject> audioStreams = filterStreamsByType(streams, AudioStream.class);
        if (!audioStreams.isEmpty()) {
            FFMpegHelper.mapAllStreams(command, audioStreams);
        }

        // Map subtitles if present
        List<StreamObject> subtitleStreams = filterStreamsByType(streams, SubtitleStream.class);
        if (!subtitleStreams.isEmpty()) {
            FFMpegHelper.mapAllStreams(command, subtitleStreams);
            command.addOutputFileOption("-c:s", "copy");
        }

        command.addOutputFileOption("-f", "matroska");

        command.setOutputFile(outFile);
        return command.getResult();
    }

}
