package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.container.FFMpegCommand;
import net.knarcraft.ffmpegconverter.container.StreamProbeResult;
import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import org.jetbrains.annotations.NotNull;

import java.util.List;

/**
 * A converter to embed adjacent subtitles into mp4 files
 */
public class SubtitleEmbed extends AbstractConverter {

    /**
     * Initializes variables used by the abstract converter
     *
     * @param ffprobePath <p>Path/command to ffprobe.</p>
     * @param ffmpegPath  <p>Path/command to ffmpeg.</p>
     */
    public SubtitleEmbed(String ffprobePath, String ffmpegPath) {
        super(null);
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
    }

    @Override
    public String[] getValidFormats() {
        return new String[]{"mp4"};
    }

    @Override
    public String[] generateConversionCommand(@NotNull String executable, @NotNull StreamProbeResult probeResult,
                                              @NotNull String outFile) {
        FFMpegCommand command = FFMpegHelper.getFFMpegGeneralFileCommand(executable, probeResult.parsedFiles());
        List<StreamObject> streams = probeResult.parsedStreams();
        
        FFMpegHelper.mapAllStreams(command, streams);
        command.addOutputFileOption("-c:a", "copy");
        command.addOutputFileOption("-c:v", "copy");
        command.addOutputFileOption("-c:s", "mov_text");

        if (this.debug) {
            FFMpegHelper.addDebugArguments(command, 50, 120);
        }

        command.setOutputFile(outFile);
        return command.getResult();
    }

}
